@@ -146,6 +146,7 @@ typedef struct _Module {
 
 	/* Registered modules */
 	char *name;
+	char *filename;
 	p11_dict *config;
 	bool critical;
 
@@ -158,7 +159,7 @@ typedef struct _Module {
 
 	/* Initialization, mutex must be held */
 	p11_mutex_t initialize_mutex;
-	bool initialize_called;
+	unsigned int initialize_called;
 	p11_thread_id_t initialize_thread;
 } Module;
 
@@ -247,7 +248,6 @@ free_module_unlocked (void *data)
 		p11_debug_precond ("module unloaded without C_Finalize having been "
 		                   "called for each C_Initialize");
 	} else {
-		assert (!mod->initialize_called);
 		assert (mod->initialize_thread == 0);
 	}
 
@@ -257,6 +257,7 @@ free_module_unlocked (void *data)
 	p11_mutex_uninit (&mod->initialize_mutex);
 	p11_dict_free (mod->config);
 	free (mod->name);
+	free (mod->filename);
 	free (mod);
 }
 
@@ -364,6 +365,8 @@ load_module_from_file_inlock (const char *name,
 	p11_debug ("loading module %s%sfrom path: %s",
 	           name ? name : "", name ? " " : "", path);
 
+	mod->filename = strdup (path);
+
 	rv = dlopen_and_get_function_list (mod, path, &funcs);
 	free (expand);
 
@@ -411,6 +414,7 @@ setup_module_for_remote_inlock (const char *name,
 		return CKR_DEVICE_ERROR;
 	}
 
+	mod->filename = NULL;
 	mod->loaded_module = rpc;
 	mod->loaded_destroy = p11_rpc_transport_free;
 
@@ -484,9 +488,7 @@ take_config_and_load_module_inlock (char **name,
 {
 	const char *filename = NULL;
 	const char *remote = NULL;
-	char *value = NULL;
 	CK_RV rv = CKR_OK;
-	bool isolated;
 	Module *mod;
 
 	assert (name);
@@ -506,18 +508,6 @@ take_config_and_load_module_inlock (char **name,
 		}
 	}
 
-	/* The 'isolated' setting is just a simple way to configure remote */
-	isolated = _p11_conf_parse_boolean (p11_dict_get (*config, "isolated"), false);
-	if (isolated) {
-		if (remote) {
-			p11_message ("ignoring 'isolated' on module '%s' because 'remote' is set", *name);
-		} else {
-			if (asprintf (&value, "|" BINDIR "/p11-kit remote '%s'", filename) < 0)
-				return_val_if_reached (CKR_DEVICE_ERROR);
-			remote = value;
-		}
-	}
-
 	if (remote != NULL) {
 		rv = setup_module_for_remote_inlock (*name, remote, &mod);
 		if (rv != CKR_OK)
@@ -547,7 +537,6 @@ take_config_and_load_module_inlock (char **name,
 	mod->critical = critical;
 
 out:
-	free (value);
 	return rv;
 }
 
@@ -646,7 +635,7 @@ initialize_module_inlock_reentrant (Module *mod)
 	p11_unlock ();
 	p11_mutex_lock (&mod->initialize_mutex);
 
-	if (!mod->initialize_called) {
+	if (mod->initialize_called != p11_forkid) {
 		p11_debug ("C_Initialize: calling");
 
 		rv = mod->virt.funcs.C_Initialize (&mod->virt.funcs,
@@ -656,10 +645,12 @@ initialize_module_inlock_reentrant (Module *mod)
 
 		/* Module was initialized and C_Finalize should be called */
 		if (rv == CKR_OK)
-			mod->initialize_called = true;
+			mod->initialize_called = p11_forkid;
+		else
+			mod->initialize_called = 0;
 
 		/* Module was already initialized, we don't call C_Finalize */
-		else if (rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
+		if (rv == CKR_CRYPTOKI_ALREADY_INITIALIZED)
 			rv = CKR_OK;
 	}
 
@@ -678,31 +669,6 @@ initialize_module_inlock_reentrant (Module *mod)
 	return rv;
 }
 
-#ifdef OS_UNIX
-
-static void
-reinitialize_after_fork (void)
-{
-	p11_dictiter iter;
-	Module *mod;
-
-	p11_debug ("forked");
-
-	p11_lock ();
-
-		if (gl.modules) {
-			p11_dict_iterate (gl.modules, &iter);
-			while (p11_dict_next (&iter, (void **)&mod, NULL))
-				mod->initialize_called = false;
-		}
-
-	p11_unlock ();
-
-	p11_proxy_after_fork ();
-}
-
-#endif /* OS_UNIX */
-
 static CK_RV
 init_globals_unlocked (void)
 {
@@ -732,9 +698,6 @@ init_globals_unlocked (void)
 	if (once)
 		return CKR_OK;
 
-#ifdef OS_UNIX
-	pthread_atfork (NULL, NULL, reinitialize_after_fork);
-#endif
 	once = true;
 
 	return CKR_OK;
@@ -790,9 +753,9 @@ finalize_module_inlock_reentrant (Module *mod)
 	p11_unlock ();
 	p11_mutex_lock (&mod->initialize_mutex);
 
-	if (mod->initialize_called) {
+	if (mod->initialize_called == p11_forkid) {
 		mod->virt.funcs.C_Finalize (&mod->virt.funcs, NULL);
-		mod->initialize_called = false;
+		mod->initialize_called = 0;
 	}
 
 	p11_mutex_unlock (&mod->initialize_mutex);
@@ -1194,6 +1157,46 @@ p11_kit_module_get_name (CK_FUNCTION_LIST *module)
 	return name;
 }
 
+/**
+ * p11_kit_module_get_filename:
+ * @module: pointer to a loaded module
+ *
+ * Get the configured name of the PKCS\#11 module.
+ *
+ * Configured modules are loaded by p11_kit_modules_load(). The module
+ * passed to this function can be either managed or unmanaged. Non
+ * configured modules will return %NULL.
+ *
+ * Use free() to release the return value when you're done with it.
+ *
+ * Returns: a newly allocated string containing the module name, or
+ *     <code>NULL</code> if the module is not a configured module
+ */
+char *
+p11_kit_module_get_filename (CK_FUNCTION_LIST *module)
+{
+	Module *mod;
+	char *name = NULL;
+
+	return_val_if_fail (module != NULL, NULL);
+
+	p11_library_init_once ();
+
+	p11_lock ();
+
+		p11_message_clear ();
+
+		if (gl.modules) {
+			mod = module_for_functions_inlock (module);
+			if (mod && mod->filename)
+				name = strdup (mod->filename);
+		}
+
+	p11_unlock ();
+
+	return name;
+}
+
 static const char *
 module_get_option_inlock (Module *mod,
                           const char *option)
@@ -1450,7 +1453,7 @@ cleanup:
 typedef struct {
 	p11_virtual virt;
 	Module *mod;
-	pid_t initialized;
+	unsigned int initialized;
 	p11_dict *sessions;
 } Managed;
 
@@ -1460,14 +1463,12 @@ managed_C_Initialize (CK_X_FUNCTION_LIST *self,
 {
 	Managed *managed = ((Managed *)self);
 	p11_dict *sessions;
-	pid_t pid;
 	CK_RV rv;
 
 	p11_debug ("in");
 	p11_lock ();
 
-	pid = getpid ();
-	if (managed->initialized == pid) {
+	if (managed->initialized == p11_forkid) {
 		rv = CKR_CRYPTOKI_ALREADY_INITIALIZED;
 
 	} else {
@@ -1480,7 +1481,7 @@ managed_C_Initialize (CK_X_FUNCTION_LIST *self,
 			rv = initialize_module_inlock_reentrant (managed->mod);
 		if (rv == CKR_OK) {
 			managed->sessions = sessions;
-			managed->initialized = pid;
+			managed->initialized = p11_forkid;
 		} else {
 			p11_dict_free (sessions);
 		}
@@ -1581,18 +1582,16 @@ managed_C_Finalize (CK_X_FUNCTION_LIST *self,
 {
 	Managed *managed = ((Managed *)self);
 	CK_SESSION_HANDLE *sessions;
-	pid_t pid;
 	int count;
 	CK_RV rv;
 
 	p11_debug ("in");
 	p11_lock ();
 
-	pid = getpid ();
 	if (managed->initialized == 0) {
 		rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 
-	} else if (managed->initialized != pid) {
+	} else if (managed->initialized != p11_forkid) {
 		/*
 		 * In theory we should be returning CKR_CRYPTOKI_NOT_INITIALIZED here
 		 * but enough callers are not completely aware of their forking.
@@ -1688,9 +1687,13 @@ managed_C_CloseAllSessions (CK_X_FUNCTION_LIST *self,
 
 	self = &managed->mod->virt.funcs;
 	managed_close_sessions (self, stolen, count);
-	free (stolen);
+	if (stolen) {
+		free (stolen);
+		return CKR_OK;
+	} else {
+		return CKR_GENERAL_ERROR;
+	}
 
-	return stolen ? CKR_OK : CKR_GENERAL_ERROR;
 }
 
 static void
