@@ -81,7 +81,8 @@ typedef struct {
 	Mapping *mappings;
 	unsigned int n_mappings;
 	p11_dict *sessions;
-	p11_array *inited;
+	CK_FUNCTION_LIST **inited;
+	unsigned int forkid;
 } Proxy;
 
 typedef struct _State {
@@ -96,6 +97,9 @@ static CK_FUNCTION_LIST **all_modules = NULL;
 static State *all_instances = NULL;
 static State global = { { { { -1, -1 }, NULL, }, }, NULL, NULL, FIRST_HANDLE, NULL };
 
+#define PROXY_VALID(px) ((px) && (px)->forkid == p11_forkid)
+#define PROXY_FORKED(px) ((px) && (px)->forkid != p11_forkid)
+
 #define MANUFACTURER_ID         "PKCS#11 Kit                     "
 #define LIBRARY_DESCRIPTION     "PKCS#11 Kit Proxy Module        "
 #define LIBRARY_VERSION_MAJOR   1
@@ -137,7 +141,7 @@ map_slot_to_real (Proxy *px,
 
 	p11_lock ();
 
-		if (!px)
+		if (!PROXY_VALID (px))
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		else
 			rv = map_slot_unlocked (px, *slot, mapping);
@@ -163,7 +167,7 @@ map_session_to_real (Proxy *px,
 
 	p11_lock ();
 
-		if (!px) {
+		if (!PROXY_VALID (px)) {
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		} else {
 			assert (px->sessions);
@@ -184,50 +188,18 @@ map_session_to_real (Proxy *px,
 }
 
 static void
-proxy_free (Proxy *py)
+proxy_free (Proxy *py, unsigned finalize)
 {
 	if (py) {
-		p11_array_free (py->inited);
+		if (finalize)
+			p11_kit_modules_finalize (py->inited);
+		free (py->inited);
 		p11_dict_free (py->sessions);
 		free (py->mappings);
 		free (py);
 	}
 }
 
-void
-p11_proxy_after_fork (void)
-{
-	p11_array *array;
-	State *state;
-	unsigned int i;
-
-	/*
-	 * After a fork the callers are supposed to call C_Initialize and all.
-	 * In addition the underlying libraries may change their state so free
-	 * up any mappings and all
-	 */
-
-	array = p11_array_new (NULL);
-
-	p11_lock ();
-
-		if (global.px)
-			p11_array_push (array, global.px);
-		global.px = NULL;
-
-		for (state = all_instances; state != NULL; state = state->next) {
-			if (state->px)
-				p11_array_push (array, state->px);
-			state->px = NULL;
-		}
-
-	p11_unlock ();
-
-	for (i = 0; i < array->num; i++)
-		proxy_free (array->elem[i]);
-	p11_array_free (array);
-}
-
 static CK_RV
 proxy_C_Finalize (CK_X_FUNCTION_LIST *self,
                   CK_VOID_PTR reserved)
@@ -246,8 +218,10 @@ proxy_C_Finalize (CK_X_FUNCTION_LIST *self,
 	} else {
 		p11_lock ();
 
-			if (!state->px) {
+			if (!PROXY_VALID (state->px)) {
 				rv = CKR_CRYPTOKI_NOT_INITIALIZED;
+				py = state->px;
+				state->px = NULL;
 			} else if (state->px->refs-- == 1) {
 				py = state->px;
 				state->px = NULL;
@@ -255,13 +229,24 @@ proxy_C_Finalize (CK_X_FUNCTION_LIST *self,
 
 		p11_unlock ();
 
-		proxy_free (py);
+		proxy_free (py, 1);
 	}
 
 	p11_debug ("out: %lu", rv);
 	return rv;
 }
 
+static CK_FUNCTION_LIST **
+modules_dup (CK_FUNCTION_LIST **modules)
+{
+	int count = 0;
+
+	while (modules[count] != NULL)
+		count++;
+
+	return memdup (modules, sizeof (CK_FUNCTION_LIST *) * (count + 1));
+}
+
 static CK_RV
 proxy_create (Proxy **res)
 {
@@ -275,53 +260,50 @@ proxy_create (Proxy **res)
 	py = calloc (1, sizeof (Proxy));
 	return_val_if_fail (py != NULL, CKR_HOST_MEMORY);
 
-	py->inited = p11_array_new ((p11_destroyer)p11_kit_module_finalize);
+	py->forkid = p11_forkid;
+
+	py->inited = modules_dup (all_modules);
 	return_val_if_fail (py->inited != NULL, CKR_HOST_MEMORY);
 
-	for (f = all_modules; *f; ++f) {
-		funcs = *f;
+	rv = p11_kit_modules_initialize (py->inited, NULL);
 
-		rv = p11_kit_module_initialize (funcs);
-		if (rv != CKR_OK)
-			break;
+	if (rv == CKR_OK) {
+		for (f = py->inited; *f; ++f) {
+			funcs = *f;
+			assert (funcs != NULL);
+			slots = NULL;
 
-		/* Make note of everything we've initialized */
-		if (!p11_array_push (py->inited, funcs))
-			return_val_if_reached (CKR_HOST_MEMORY);
+			/* Ask module for its slots */
+			rv = (funcs->C_GetSlotList) (FALSE, NULL, &count);
+			if (rv == CKR_OK && count) {
+				slots = calloc (sizeof (CK_SLOT_ID), count);
+				rv = (funcs->C_GetSlotList) (FALSE, slots, &count);
+			}
 
-		assert (funcs != NULL);
-		slots = NULL;
+			if (rv != CKR_OK) {
+				free (slots);
+				break;
+			}
 
-		/* Ask module for its slots */
-		rv = (funcs->C_GetSlotList) (FALSE, NULL, &count);
-		if (rv == CKR_OK && count) {
-			slots = calloc (sizeof (CK_SLOT_ID), count);
-			rv = (funcs->C_GetSlotList) (FALSE, slots, &count);
-		}
+			return_val_if_fail (count == 0 || slots != NULL, CKR_GENERAL_ERROR);
+
+			py->mappings = realloc (py->mappings, sizeof (Mapping) * (py->n_mappings + count));
+			return_val_if_fail (py->mappings != NULL, CKR_HOST_MEMORY);
+
+			/* And now add a mapping for each of those slots */
+			for (i = 0; i < count; ++i) {
+				py->mappings[py->n_mappings].funcs = funcs;
+				py->mappings[py->n_mappings].wrap_slot = py->n_mappings + MAPPING_OFFSET;
+				py->mappings[py->n_mappings].real_slot = slots[i];
+				++py->n_mappings;
+			}
 
-		if (rv != CKR_OK) {
 			free (slots);
-			break;
 		}
-
-		return_val_if_fail (count == 0 || slots != NULL, CKR_GENERAL_ERROR);
-
-		py->mappings = realloc (py->mappings, sizeof (Mapping) * (py->n_mappings + count));
-		return_val_if_fail (py->mappings != NULL, CKR_HOST_MEMORY);
-
-		/* And now add a mapping for each of those slots */
-		for (i = 0; i < count; ++i) {
-			py->mappings[py->n_mappings].funcs = funcs;
-			py->mappings[py->n_mappings].wrap_slot = py->n_mappings + MAPPING_OFFSET;
-			py->mappings[py->n_mappings].real_slot = slots[i];
-			++py->n_mappings;
-		}
-
-		free (slots);
 	}
 
 	if (rv != CKR_OK) {
-		proxy_free (py);
+		proxy_free (py, 1);
 		return rv;
 	}
 
@@ -350,10 +332,18 @@ proxy_C_Initialize (CK_X_FUNCTION_LIST *self,
 
 	p11_lock ();
 
-		if (state->px == NULL)
+		if (!PROXY_VALID (state->px)) {
+			unsigned call_finalize = 1;
+
 			initialize = true;
-		else
+			if (PROXY_FORKED(state->px))
+				call_finalize = 0;
+			proxy_free (state->px, call_finalize);
+
+			state->px = NULL;
+		} else {
 			state->px->refs++;
+		}
 
 	p11_unlock ();
 
@@ -377,7 +367,7 @@ proxy_C_Initialize (CK_X_FUNCTION_LIST *self,
 
 	p11_unlock ();
 
-	proxy_free (py);
+	proxy_free (py, 1);
 	p11_debug ("out: 0");
 	return rv;
 }
@@ -395,7 +385,7 @@ proxy_C_GetInfo (CK_X_FUNCTION_LIST *self,
 
 	p11_lock ();
 
-		if (!state->px)
+		if (!PROXY_VALID (state->px))
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 
 	p11_unlock ();
@@ -425,13 +415,13 @@ proxy_C_GetSlotList (CK_X_FUNCTION_LIST *self,
 	Mapping *mapping;
 	CK_ULONG index;
 	CK_RV rv = CKR_OK;
-	int i;
+	unsigned int i;
 
 	return_val_if_fail (count != NULL, CKR_ARGUMENTS_BAD);
 
 	p11_lock ();
 
-		if (!state->px) {
+		if (!PROXY_VALID (state->px)) {
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		} else {
 			index = 0;
@@ -579,7 +569,7 @@ proxy_C_OpenSession (CK_X_FUNCTION_LIST *self,
 	if (rv == CKR_OK) {
 		p11_lock ();
 
-			if (!state->px) {
+			if (!PROXY_VALID (state->px)) {
 				/*
 				 * The underlying module should have returned an error, so this
 				 * code should never be reached with properly behaving modules.
@@ -643,7 +633,7 @@ proxy_C_CloseAllSessions (CK_X_FUNCTION_LIST *self,
 
 	p11_lock ();
 
-		if (!state->px) {
+		if (!PROXY_VALID (state->px)) {
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		} else {
 			assert (state->px->sessions != NULL);
