@@ -55,6 +55,7 @@
 #include <string.h>
 
 #ifdef OS_UNIX
+#include <sys/select.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
 #include <sys/un.h>
@@ -64,6 +65,9 @@
 
 #ifdef OS_WIN32
 #include <winsock2.h>
+#ifndef EWOULDBLOCK
+#define EWOULDBLOCK WSAEWOULDBLOCK
+#endif
 #endif
 
 #ifndef EPROTO
@@ -360,8 +364,10 @@ rpc_socket_read (rpc_socket *sock,
 	p11_mutex_lock (&sock->read_lock);
 
 	if (!sock->read_creds) {
-		if (read_all (sock->fd, &dummy, 1) != 1)
+		if (read_all (sock->fd, &dummy, 1) != 1) {
+			p11_mutex_unlock (&sock->read_lock);
 			return CKR_DEVICE_ERROR;
+		}
 		sock->read_creds = true;
 	}
 
