@@ -38,14 +38,16 @@
 #include "compat.h"
 #include "debug.h"
 #include "extract.h"
-#include "iter.h"
 #include "message.h"
 #include "oid.h"
 #include "path.h"
-#include "pkcs11.h"
 #include "pkcs11x.h"
 #include "save.h"
 #include "tool.h"
+#include "digest.h"
+
+#include "p11-kit/iter.h"
+#include "p11-kit/pkcs11.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -76,6 +78,7 @@ format_argument (const char *optarg,
 		{ "x509-directory", p11_extract_x509_directory, },
 		{ "pem-bundle", p11_extract_pem_bundle, },
 		{ "pem-directory", p11_extract_pem_directory },
+		{ "pem-directory-hash", p11_extract_pem_directory_hash },
 		{ "java-cacerts", p11_extract_jks_cacerts },
 		{ "openssl-bundle", p11_extract_openssl_bundle },
 		{ "openssl-directory", p11_extract_openssl_directory },
@@ -197,13 +200,14 @@ p11_trust_extract (int argc,
 		},
 		{ opt_format,
 		  "format to extract to\n"
-		  "  x509-file         DER X.509 certificate file\n"
-		  "  x509-directory    directory of X.509 certificates\n"
-		  "  pem-bundle        file containing multiple PEM blocks\n"
-		  "  pem-directory     directory of PEM files\n"
-		  "  openssl-bundle    OpenSSL specific PEM bundle\n"
-		  "  openssl-directory directory of OpenSSL specific files\n"
-		  "  java-cacerts      java keystore cacerts file",
+		  "  x509-file           DER X.509 certificate file\n"
+		  "  x509-directory      directory of X.509 certificates\n"
+		  "  pem-bundle          file containing multiple PEM blocks\n"
+		  "  pem-directory       directory of PEM files\n"
+		  "  pem-directory-hash  directory of PEM files with hash links\n"
+		  "  openssl-bundle      OpenSSL specific PEM bundle\n"
+		  "  openssl-directory   directory of OpenSSL specific files\n"
+		  "  java-cacerts        java keystore cacerts file",
 		  "type"
 		},
 		{ opt_purpose,
@@ -289,7 +293,6 @@ p11_trust_extract_compat (int argc,
                           char *argv[])
 {
 	char *path = NULL;
-	char *path2 = NULL;
 	int error;
 
 	argv[argc] = NULL;
@@ -298,24 +301,22 @@ p11_trust_extract_compat (int argc,
 	 * For compatibility with people who deployed p11-kit 0.18.x
 	 * before trust stuff was put into its own branch.
 	 */
-	path2 = p11_path_build (PRIVATEDIR, "p11-kit-extract-trust", NULL);
-	return_val_if_fail (path2 != NULL, 1);
-	execv (path2, argv);
+	path = p11_path_build (PRIVATEDIR, "p11-kit-extract-trust", NULL);
+	return_val_if_fail (path != NULL, 1);
+	execv (path, argv);
 	error = errno;
-	free (path2);
 
 	if (error == ENOENT) {
+		free (path);
 		path = p11_path_build (PRIVATEDIR, "trust-extract-compat", NULL);
 		return_val_if_fail (path != NULL, 1);
 		execv (path, argv);
 		error = errno;
-		free (path);
 	}
 
 	/* At this point we have no command */
 	p11_message_err (error, "could not run %s command", path);
 
 	free (path);
-	free (path2);
 	return 2;
 }
