@@ -145,7 +145,7 @@ p11_save_write (p11_save_file *file,
 	while (written < length) {
 		res = write (file->fd, buf + written, length - written);
 		if (res <= 0) {
-			if (errno == EAGAIN && errno == EINTR)
+			if (errno == EAGAIN || errno == EINTR)
 				continue;
 			p11_message_err (errno, "couldn't write to file: %s", file->temp);
 			return false;
@@ -233,7 +233,6 @@ p11_save_finish_file (p11_save_file *file,
 	/* Set the mode of the file, readable by everyone, but not writable */
 	} else if (chmod (file->temp, S_IRUSR | S_IRGRP | S_IROTH) < 0) {
 		p11_message_err (errno, "couldn't set file permissions: %s", file->temp);
-		close (file->fd);
 		ret = false;
 
 	/* Atomically rename the tempfile over the filename */
